download <- function(input_gse, annot_column , annot_type, input_gpl = NA) {
    Sys.setenv("VROOM_CONNECTION_SIZE"=99999999)
    # source helper functions
    source(paste(path, "/GEO_array_helper.R", sep =""))
    source(paste(path, "/convert_gct.R", sep =""))
    message("INFO: GSE ID: ", input_gse)
    message("INFO: GPL ID: ", input_gpl)

    if(is.null(input_gse)|| input_gse=="")
        stop("ERROR: no input gse_id provided")

    # define stop messages
    stop_messages <- list(
        "low_genes" = "No rows in the matrix,GEOQuery did not get data properly",
        "NA_genes" = "All rows contain columns with NAs",
        "platform_annotation" = "Not able to annotate platform",
        "platform_not_MA" = "platform not a Microrarray platform or not in annotations file",
        "annot_not_defined" = "Annotation is not defined in platform annotation file",
        "entrez_col_no_digit" = "Entrez column does not contain integers in platform annotation file",
        "failure" = "failed"
      )
    # get GEO object
    gse_gpl_object <- download_geo(input_gse, input_gpl)

    #get expression matrix
    gse_matrix <- gse_gpl_object@assayData$exprs
    message("INFO: ", "Dimensions of GEO matrix: ",
        paste0(dim(gse_matrix)[1], "x", dim(gse_matrix)[2]))

    #check if gse matrix is Empty
    if (nrow(gse_matrix)==0)
        stop("ERROR: GSE matrix is empty")

    # get platform object for annotation
    gpl <- gse_gpl_object@annotation

    # get metadata dataframe
    gse_metadata <- gse_gpl_object@phenoData@data
    message("INFO: ", "Dimensions of GEO metadata: ",
        paste0(dim(gse_metadata)[1], "x", dim(gse_metadata)[2]))

    #annotate geo matrix
    gse_matrix <- annotate_gse_matrix(gse_matrix, gpl, annot_column, annot_type)
    message("INFO: GSE matrix dims after annotation: ",
        paste0(dim(gse_matrix)[1], "x", dim(gse_matrix)[2]))
    if (nrow(gse_matrix) == 0)
    stop("ERROR: ", stop_messages[["entrez_col_no_digit"]])

    message("INFO: Remove rows with NAs")
    columns_to_exclude <- c("probe_id", "entrez")
    # Check for NAs in all columns except the specified ones
    condition <- apply(gse_matrix[, !colnames(gse_matrix) %in% columns_to_exclude], 1, function(row) {
      anyNA(row) || any(row == "")
    })
    # Use subset to keep rows that don't meet the condition
    gse_matrix <- subset(gse_matrix, !condition)
    message("INFO: GSE matrix dims after NA removal: ",
        paste0(dim(gse_matrix)[1], "x", dim(gse_matrix)[2]))
    if (nrow(gse_matrix) == 0)
        stop("ERROR: ", stop_messages[["NA_genes"]])

    message("INFO: Checking if matrix is log-transformed")
    log2_tranformed <- check_log2(gse_matrix)

    if (!log2_tranformed) {
        message("INFO: matrix is not log2 tranformed. So transforming it")
        gse_matrix <- tranform_log2(gse_matrix)
    }

     # check set equality of colnames of matrix and row names of metadata
    if(!setequal(colnames(gse_matrix), rownames(gse_metadata))){
        missing_samples <- c(setdiff(colnames(gse_matrix), rownames(gse_metadata)),
                             setdiff(rownames(gse_metadata), colnames(gse_matrix)))
        if (length(missing_samples[!(missing_samples %in% c('probe_id','entrez'))])>0)
            stop(paste0("ERROR: samples ", paste(missing_samples, collapse = ", "),
                    "are not present in both matrix and metadata"))
    }

    # Add all columns from GPL table to gse_matrix
    gse_matrix<-add_all_cols_gpl(gse_matrix,gpl)
    message("INFO: Completed conversion of GEO dataset")
    
    message("INFO: Getting study metadata")
    study_meta_list <- study_metadata_from_geo_object_list(gse_gpl_object, gpl)
    
    output_filepath <- paste0(input_gse, "_", gpl, ".gct")
    gse_metadata_char  <- data.frame(lapply(gse_metadata, as.character), stringsAsFactors=FALSE)
    rownames(gse_metadata_char) <- rownames(gse_metadata)

    gse_matrix_df <- as.data.frame(gse_matrix)
    gct_filepath <- convert_gct(gse_matrix_df, gse_metadata_char, output_filepath)
    message("GCT file saved at: ", gct_filepath)
    message("data matrix", head(gse_matrix_df))
    return(c(gct_filepath = gct_filepath,gse_matrix = gse_matrix_df,gse_metadata_char=gse_metadata_char,study_meta_list = study_meta_list))
    }